// node_modules/.pnpm/data-uri-to-buffer@4.0.1/node_modules/data-uri-to-buffer/dist/index.js
function dataUriToBuffer(uri) {
  if (!/^data:/i.test(uri)) {
    throw new TypeError('`uri` does not appear to be a Data URI (must begin with "data:")');
  }
  uri = uri.replace(/\r?\n/g, "");
  const firstComma = uri.indexOf(",");
  if (firstComma === -1 || firstComma <= 4) {
    throw new TypeError("malformed data: URI");
  }
  const meta = uri.substring(5, firstComma).split(";");
  let charset = "";
  let base64 = false;
  const type = meta[0] || "text/plain";
  let typeFull = type;
  for (let i = 1; i < meta.length; i++) {
    if (meta[i] === "base64") {
      base64 = true;
    } else if (meta[i]) {
      typeFull += `;${meta[i]}`;
      if (meta[i].indexOf("charset=") === 0) {
        charset = meta[i].substring(8);
      }
    }
  }
  if (!meta[0] && !charset.length) {
    typeFull += ";charset=US-ASCII";
    charset = "US-ASCII";
  }
  const encoding = base64 ? "base64" : "ascii";
  const data = unescape(uri.substring(firstComma + 1));
  const buffer = Buffer.from(data, encoding);
  buffer.type = type;
  buffer.typeFull = typeFull;
  buffer.charset = charset;
  return buffer;
}
var dist_default = dataUriToBuffer;

// src/fastify-fetch.ts
import fp from "fastify-plugin";
import {
  fetch,
  Headers,
  Request,
  Response
} from "undici";
import { promisify } from "util";
import {
  brotliDecompress as _brotliDecompress,
  gunzip as _gunzip,
  inflate as _inflate
} from "zlib";
var gunzip = promisify(_gunzip);
var brotliDecompress = promisify(_brotliDecompress);
var inflate = promisify(_inflate);
var supportedSchemas = /* @__PURE__ */ new Set(["data:", "http:", "https:"]);
function splitCookiesString(cookiesString) {
  const cookiesStrings = [];
  let pos = 0;
  let start;
  let ch;
  let lastComma;
  let nextStart;
  let cookiesSeparatorFound;
  function skipWhitespace() {
    while (pos < cookiesString.length && /\s/.test(cookiesString.charAt(pos))) {
      pos += 1;
    }
    return pos < cookiesString.length;
  }
  function notSpecialChar() {
    ch = cookiesString.charAt(pos);
    return ch !== "=" && ch !== ";" && ch !== ",";
  }
  while (pos < cookiesString.length) {
    start = pos;
    cookiesSeparatorFound = false;
    while (skipWhitespace()) {
      ch = cookiesString.charAt(pos);
      if (ch === ",") {
        lastComma = pos;
        pos += 1;
        skipWhitespace();
        nextStart = pos;
        while (pos < cookiesString.length && notSpecialChar()) {
          pos += 1;
        }
        if (pos < cookiesString.length && cookiesString.charAt(pos) === "=") {
          cookiesSeparatorFound = true;
          pos = nextStart;
          cookiesStrings.push(cookiesString.substring(start, lastComma));
          start = pos;
        } else {
          pos = lastComma + 1;
        }
      } else {
        pos += 1;
      }
    }
    if (!cookiesSeparatorFound || pos >= cookiesString.length) {
      cookiesStrings.push(cookiesString.substring(start, cookiesString.length));
    }
  }
  return cookiesStrings;
}
function fromNodeHeaders(object) {
  const headers = new Headers();
  for (const [key, value] of Object.entries(object)) {
    const values = Array.isArray(value) ? value : [value];
    for (const v of values) {
      if (v !== void 0) {
        headers.append(key, `${v}`);
      }
    }
  }
  return headers;
}
function toNodeHeaders(headers) {
  const result = {};
  if (headers != null) {
    for (const [key, value] of headers.entries()) {
      result[key] = value;
      if (key.toLowerCase() === "set-cookie") {
        result[key] = splitCookiesString(value);
      }
    }
  }
  return result;
}
var decompressPayload = async (rawPayload, headers) => {
  if (rawPayload.length === 0) {
    return void 0;
  }
  const codings = headers.get("content-encoding")?.toLowerCase().split(",").map((x) => x.trim()) ?? [];
  if (codings.length === 0) {
    return rawPayload;
  }
  const decoders = [];
  const remainingCodings = codings.map((coding) => {
    if (/(x-)?gzip/.test(coding)) {
      decoders.push(async (value) => await gunzip(value));
    } else if (/(x-)?deflate/.test(coding)) {
      decoders.push(async (value) => await inflate(value));
    } else if (coding === "br") {
      decoders.push(async (value) => await brotliDecompress(value));
    } else {
      return coding;
    }
    return void 0;
  }).filter((value) => value !== void 0);
  decoders.push(async (value) => await Promise.resolve(value));
  const payload = await decoders.reduce(
    (previousValue, currentValue) => async (value) => await currentValue(await previousValue(value))
  )(rawPayload);
  if (remainingCodings.length === 0) {
    headers.delete("content-encoding");
  } else {
    headers.set("content-encoding", Array.from(remainingCodings).join(", "));
  }
  return payload;
};
var fastifyFetch = fp(async (app, options = {}) => {
  const hasMatchFunction = typeof options.match === "function";
  app.decorate(
    "fetch",
    async (requestInfo, requestInit) => {
      const request = new Request(requestInfo, requestInit);
      const parsedURL = new URL(request.url);
      if (!supportedSchemas.has(parsedURL.protocol)) {
        throw new TypeError(
          `URL scheme "${parsedURL.protocol.replace(
            /:$/,
            ""
          )}" is not supported.`
        );
      }
      if (parsedURL.protocol === "data:") {
        const data = dist_default(request.url);
        return new Response(data, {
          headers: { "Content-Type": data.typeFull }
        });
      }
      const method = request.method.toUpperCase();
      if (!["DELETE", "GET", "HEAD", "OPTIONS", "PATCH", "POST", "PUT"].includes(
        method
      )) {
        throw new TypeError(`${method} is not supported.`);
      }
      if (!hasMatchFunction || // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
      hasMatchFunction && options.match(parsedURL, request)) {
        const response = await app.inject({
          url: request.url,
          headers: toNodeHeaders(request.headers),
          payload: Buffer.from(await request.arrayBuffer()),
          method
        });
        const headers = fromNodeHeaders(response.headers);
        const statusText = response.statusMessage;
        const status = response.statusCode;
        const payload = await decompressPayload(response.rawPayload, headers);
        return new Response(payload, {
          headers,
          statusText,
          status
        });
      } else {
        return await (options.fetch ?? fetch)(request);
      }
    }
  );
});

// src/index.ts
var src_default = fastifyFetch;
export {
  src_default as default,
  fastifyFetch
};
//# sourceMappingURL=index.mjs.map
